/*************************************************************************
*
*  Copyright (C) 1993 IBM Corporation
*
*      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
*      sample code created by IBM Corporation. This sample code is not
*      part of any standard or IBM product and is provided to you solely
*      for  the purpose of assisting you in the development of your
*      applications.  The code is provided "AS IS", without
*      warranty of any kind.  IBM shall not be liable for any damages
*      arising out of your use of the sample code, even if they have been
*      advised of the possibility of such damages.
*
************************************************************************/
/*
        CALCFUNC.CPP contains the code for doing the following:
        1.      Declaring and defining the operations supported by the
                Command Line Calculator.
        2.      Adding those functions to the calculator (InitCalcFuncs).
*/

#include <iomanip.h>
#include <iostream.h>
#include <math.h>
#include <stdlib.h>

#include "calc.h"
#include "calc_cls.h"

CALCULATOR CLC(0);

double Input = 0.0;
double Result = 0.0;

int InitCalcFuncs(void);

int Add(void);
int Div(void);
int End(void);
int Help(void);
int Inv(void);
int Mult(void);
int Recall(void);
int ShowRegisters(void);
// int Square(void)
int Sqrt(void);
int Stats(void);
int Store(void);
int Sub(void);

int InitCalcFuncs()
{
        // Initialize the registers.  The first 8 (0 to 7) are read/write.
        // The last 8 (8 to 15) are read-only.


        CLC.AddFunction("add", "Adds x to result.  Syntax: \"add x\".", Add);
        CLC.AddFunction("div", "Divides result by x.  Syntax: \"div x\".", Div);
        CLC.AddFunction("end", "Ends CLC.  Syntax: \"end\".", End);
        CLC.AddFunction("help", "Lists this help.  Syntax: \"help\".", Help);
        CLC.AddFunction("inv", "Inverts result.  Syntax: \"inv\".", Inv);
        CLC.AddFunction("mult", "Multiplies result by x.  Syntax: \"mult x\".", Mult);
        CLC.AddFunction("rcl", "Copies value of register x into result.  Syntax: \"rcl x\".", Recall);
        CLC.AddFunction("regs", "Show contents of registers.  Syntax: \"regs\".", ShowRegisters);
        CLC.AddFunction("sqrt", "Sets result to its square root.  Syntax \"sqrt\".", Sqrt);
//        CLC.AddFunction("square", "Squares result.  Syntax: \"square\".", Square);
        CLC.AddFunction("stats",
                     "Finds average and standard deviation of a set of values.  \n\tSyntax: \"stats\".",
                     Stats);
        CLC.AddFunction("sto", "Copies result into register x.  Syntax: \"sto x\".", Store);
        CLC.AddFunction("sub", "Subtracts x from result.  Syntax: \"sub x\".", Sub);

        CLC.ShowHelp();

        return NO_ERROR;
}

// Add the input value
int Add(void)
{
        Result += Input;
        return NO_ERROR;
}

int Div(void)
{
        if ((Input * Input) <= 0.0001)
        {
                cout << "\nInput is too small for division." << endl;
                return ERROR;
        }

        Result /= Input;
        return NO_ERROR;
}

int End(void)
{
        return END_PROGRAM;
}

int Help(void)
{
        CLC.ShowHelp();

        return 0;
}

int Inv(void)
{
        if ((Result * Result) <= 0.0001)
        {
                cout << "\nResult is too small for inversion." << endl;
                return ERROR;
        }

        Result = 1.0/Result;
        return NO_ERROR;
}

int Mult(void)
{
        Result *= Input;
        return NO_ERROR;
}

int Recall(void)
{
        int iInput = (int) Input;
        if ((iInput >= 0) && (iInput <= 15))
                Result = CLC.Reg[iInput].GetValue();

        return NO_ERROR;
}

int ShowRegisters(void)
{
        cout << "..........................................................." << endl;
        cout << "USER REGISTERS" << endl;
        cout << "--------------" << endl;

        for( int i = 0, j = 3; i <= 15; i++)
        {
                if (i == 8)
                {
                        cout << "PROGRAM REGISTERS" << endl;
                        cout << "-----------------" << endl;
                }
                cout << setw(2) << i << ":[" << setw(10) << CLC.Reg[i].GetValue() << setw(3) << "]  ";
                if (j == 0)
                {
                        cout << endl;
                        j = 3;
                        continue;
                }
                j--;
        }
        cout << "..........................................................." << endl;

        return NO_ERROR;
}

int Sqrt(void)
{
        if (Result < 0.0)
        {
                cout << "\nCannot take square root of a negative number." << flush;
                return ERROR;
        }

        Result = sqrt(Result);

        return NO_ERROR;
}

/*
int Square(void)
{
        Rsult *= Result;

        return NO_ERROR;
}
*/

int Stats(void)
{
        int N;
        char Line[88];
        double Sum, Average = 0.0 , StdDev = 0.0, Data[16];

        cout << "\nEnter first value (maximum is 16 values): " << flush;

        int i = 0;
        cin.getline (Line, 88, '\n');

        if (*Line == NULL)
        {
                cout << "\nNo value entered so statistics function is ending." << endl;
                return NO_ERROR;
        }
        else
                Data[i] = atof( Line);

        for ( i = 1; i < 16; i++)
        {
                cout << "Enter next value (blank to finish): " << flush;
                cin.getline (Line, 88, '\n');
                if (*Line == NULL)
                        break;
                else
                        Data[i] = atof( Line);
        }
        N = i;

        for (i = 0; i < N; i++)
                Average += Data[i];

        Average /= N;

        for (i = 0; i < N; i++)
                StdDev += (Data[i] - Average) * (Data[i] - Average);

        StdDev /= N;
        StdDev = sqrt( StdDev);

        cout << "\nNumber of values is " << N;
        cout << "\nAverage is " << Average << " and is stored in register 8.";
        cout << "\nStandard Deviation is " << StdDev << " and is stored in register 9.\n" << flush;

        CLC.Reg[8].Store( Average);
        CLC.Reg[9].Store( StdDev);

        return NO_ERROR;
}

int Store(void)
{
        int iInput = (int) Input;

        if ((iInput >= 0) && (iInput <= 7))
        {
                if (CLC.Reg[iInput].IsReadOnly() == 0)
                        CLC.Reg[iInput].Store(Result);
                else
                        cout << "You cannot store a value into a read-only register." << endl
                             << "Enter the command again with another register number." << endl;
        }
        else
                cout << "The user registers are numbered from 0 to 7." << endl
                     << "Specify a register within this range." << endl;

        return NO_ERROR;
}

int Sub(void)
{
        Result -= Input;
        return NO_ERROR;
}
